package de.w665.pr.statemachine.trafficlight;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.statemachine.action.Action;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineConfigurerAdapter;
import org.springframework.statemachine.config.builders.StateMachineStateConfigurer;
import org.springframework.statemachine.config.builders.StateMachineTransitionConfigurer;

import java.util.EnumSet;

@Slf4j
@RequiredArgsConstructor
@Configuration
@EnableStateMachine
public class TrafficLightStateMachineConfig extends StateMachineConfigurerAdapter<TrafficLightStates, TrafficLightEvents> {

    private final MinimumTimePassedGuard minimumTimePassedGuard;
    private final TimingContext timingContext;
    private final TrafficLightPersistanceService trafficLightPersistanceService;

    @Override
    public void configure(StateMachineStateConfigurer<TrafficLightStates, TrafficLightEvents> states) throws Exception{
        states
                .withStates()
                .initial(TrafficLightStates.RED)
                .state(TrafficLightStates.RED, redEntryAction(), redExitAction())
                .state(TrafficLightStates.YELLOW, yellowEntryAction(), yellowExitAction())
                .state(TrafficLightStates.GREEN, greenEntryAction(), greenExitAction());
                /*.states(EnumSet.allOf(TrafficLightStates.class));*/ // Redundant -> Only used when all enum values should be added as states | We're defining them above already
    }

    @Override
    public void configure(StateMachineTransitionConfigurer<TrafficLightStates, TrafficLightEvents> transitions) throws Exception {
        transitions
                .withExternal()
                    .source(TrafficLightStates.RED)
                    .target(TrafficLightStates.GREEN)
                    .event(TrafficLightEvents.TIMER)
                    .guard(minimumTimePassedGuard)
                    .action(recordTransitionAction())
                .and()
                .withExternal()
                    .source(TrafficLightStates.GREEN)
                    .target(TrafficLightStates.YELLOW)
                    .event(TrafficLightEvents.TIMER)
                    .guard(minimumTimePassedGuard)
                    .action(recordTransitionAction())
                .and()
                .withExternal()
                    .source(TrafficLightStates.YELLOW)
                    .target(TrafficLightStates.RED)
                    .event(TrafficLightEvents.TIMER)
                    .guard(minimumTimePassedGuard)
                    .action(recordTransitionAction());
    }

    @Bean
    public Action<TrafficLightStates, TrafficLightEvents> recordTransitionAction() {
        return context -> {
            TrafficLightStates targetState = context.getTarget().getId();
            trafficLightPersistanceService.persistTrafficLightState(targetState);
            timingContext.recordTransition();
        };
    }

    @Bean
    public Action<TrafficLightStates, TrafficLightEvents> redEntryAction() {
        return context -> log.info("Red light ON");
    }

    @Bean
    Action<TrafficLightStates, TrafficLightEvents> redExitAction() {
        return context -> log.info("Red light OFF");
    }

    @Bean
    Action<TrafficLightStates, TrafficLightEvents> yellowEntryAction() {
        return context -> log.info("Yellow light ON");
    }

    @Bean
    Action<TrafficLightStates, TrafficLightEvents> yellowExitAction() {
        return context -> log.info("Yellow light OFF");
    }

    @Bean
    Action<TrafficLightStates, TrafficLightEvents> greenEntryAction() {
        return context -> log.info("Green light ON");
    }

    @Bean
    Action<TrafficLightStates, TrafficLightEvents> greenExitAction() {
        return context -> log.info("Green light OFF");
    }

}
